package ascensore.model;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.OptionalInt;
import java.util.Queue;

public class AscensoreEvoluto extends Ascensore {

	private Queue<Integer> prenotazioni = new LinkedList<Integer>();

	public AscensoreEvoluto(int pianoMin, int pianoMax, int pianoCorrente) {
		super(pianoMin, pianoMax, pianoCorrente);
	}

	@Override
	public String getTipo() {
		return "Evoluto";
	}

	@Override
	public boolean vaiAlPiano(int pianoArrivo) {
		if (pianoArrivo > pianoMax || pianoArrivo < pianoMin)
			throw new IllegalArgumentException("piano di arrivo fuori range");
		
		if (prenotazioni.contains(pianoArrivo) || 
				// Check necessario non suggerito nel testo
				getStatoAscensore() == StatoAscensore.Fermo && getPianoCorrente() == pianoArrivo && 
				getPianoCorrente() == getPianoRichiesto())
			return false;
		
		if (prenotazioni.isEmpty()) {
			setPianoRichiesto(pianoArrivo);
		}
		prenotazioni.add(pianoArrivo);
		return true;
	}
	
	
	public void arrivatoAlPianoGestisciPrenotazioni(){
		// Nell'ascensore evoluto elimina la prenotazione corrente e imposta la successiva
		eliminaPrenotazionePianoCorrente();
		OptionalInt prossimaPrenotazione = getProssimaPrenotazione();
		if (prossimaPrenotazione.isPresent()) {
			setPianoRichiesto(prossimaPrenotazione.getAsInt());
		}
	}

	private void eliminaPrenotazionePianoCorrente() {
		int pianoPrenotato = prenotazioni.poll();
		if (pianoPrenotato != getPianoCorrente())
			throw new IllegalStateException("pianoPrenotato != getPianoCorrente()");
	}

	private OptionalInt getProssimaPrenotazione() {
		return prenotazioni.isEmpty() ? OptionalInt.empty() : OptionalInt.of(prenotazioni.peek());
	}

	@Override
	public List<Integer> getPrenotazioni() {
		return new ArrayList<Integer>(prenotazioni);
	}

}
